@@ -178,6 +178,9 @@ pub enum ConfigParseErrorKind {
         /// The features that were not enabled.
         missing_features: BTreeSet<ConfigExperimental>,
     },
+    /// An inheritance cycle was detected in the profile configuration.
+    #[error("inheritance error(s) detected: {}", .0.iter().join(", "))]
+    InheritanceErrors(Vec<InheritsError>),
 }
 
 /// An error that occurred while compiling overrides or scripts specified in
@@ -1822,6 +1825,25 @@ pub enum ShowTestGroupsError {
     },
 }
 
+/// An error occurred while processing profile's inherits setting
+#[derive(Debug, Error, PartialEq, Eq, Hash)]
+pub enum InheritsError {
+    /// The default profile should not be able to inherit from other profiles
+    #[error("the {} profile should not inherit from other profiles", .0)]
+    DefaultProfileInheritance(String),
+    /// An unknown/unfound profile was detected to inherit from in profile configuration
+    #[error("profile {} inherits from an unknown profile {}", .0, .1)]
+    UnknownInheritance(String, String),
+    /// A self referential inheritance is detected from this profile
+    #[error("a self referential inheritance is detected from profile: {}", .0)]
+    SelfReferentialInheritance(String),
+    /// An inheritance cycle was detected in the profile configuration.
+    #[error("inheritance cycle detected in profile configuration from: {}", .0.iter().map(|scc| {
+        format!("[{}]", scc.iter().join(", "))
+    }).join(", "))]
+    InheritanceCycle(Vec<Vec<String>>),
+}
+
 #[cfg(feature = "self-update")]
 mod self_update_errors {
     use super::*;
