@@ -23,7 +23,7 @@
 //! using before
 
 use crate::{
-    config::elements::LeakTimeoutResult,
+    config::elements::{LeakTimeoutResult, SlowTimeoutResult},
     errors::{DisplayErrorChain, FormatVersionError, FormatVersionErrorInner, WriteEventError},
     list::RustTestSuite,
     reporter::events::{ExecutionResult, StressIndex, TestEvent, TestEventKind},
@@ -268,6 +268,9 @@ impl<'cfg> LibtestReporter<'cfg> {
                     KIND_TEST,
                     match run_statuses.last_status().result {
                         ExecutionResult::Pass
+                        | ExecutionResult::Timeout {
+                            result: SlowTimeoutResult::Pass,
+                        }
                         | ExecutionResult::Leak {
                             result: LeakTimeoutResult::Pass,
                         } => EVENT_OK,
@@ -276,7 +279,9 @@ impl<'cfg> LibtestReporter<'cfg> {
                         }
                         | ExecutionResult::Fail { .. }
                         | ExecutionResult::ExecFail
-                        | ExecutionResult::Timeout => EVENT_FAILED,
+                        | ExecutionResult::Timeout {
+                            result: SlowTimeoutResult::Fail,
+                        } => EVENT_FAILED,
                     },
                     stress_index,
                     test_instance,
@@ -300,10 +305,10 @@ impl<'cfg> LibtestReporter<'cfg> {
         let mut test_suite = match self.test_suites.entry(&suite_info.binary_id) {
             id_ord_map::Entry::Vacant(e) => {
                 let (running, ignored, filtered) =
-                    suite_info.status.test_cases().fold((0, 0, 0), |acc, tc| {
-                        if tc.1.ignored {
+                    suite_info.status.test_cases().fold((0, 0, 0), |acc, case| {
+                        if case.test_info.ignored {
                             (acc.0, acc.1 + 1, acc.2)
-                        } else if tc.1.filter_match.is_match() {
+                        } else if case.test_info.filter_match.is_match() {
                             (acc.0 + 1, acc.1, acc.2)
                         } else {
                             (acc.0, acc.1, acc.2 + 1)
@@ -425,7 +430,9 @@ impl<'cfg> LibtestReporter<'cfg> {
                         )?;
                         out.extend_from_slice(b"\"");
                     }
-                    ExecutionResult::Timeout => {
+                    ExecutionResult::Timeout {
+                        result: SlowTimeoutResult::Fail,
+                    } => {
                         test_suite_mut.failed += 1;
                         out.extend_from_slice(br#","reason":"time limit exceeded""#);
                     }
