@@ -14,7 +14,7 @@ use nextest_metadata::{FilterMatch, MismatchReason};
 use nextest_runner::{
     config::{
         core::NextestConfig,
-        elements::{LeakTimeoutResult, RetryPolicy},
+        elements::{LeakTimeoutResult, RetryPolicy, SlowTimeoutResult},
     },
     double_spawn::DoubleSpawnInfo,
     input::InputHandlerKind,
@@ -71,6 +71,183 @@ fn test_list_binaries() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn test_timeout_with_retries() -> Result<()> {
+    test_init();
+    let pcx = ParseContext::new(&PACKAGE_GRAPH);
+    let expr = Filterset::parse(
+        "test(/^test_slow_timeout/)".to_owned(),
+        &pcx,
+        FiltersetKind::Test,
+    )
+    .unwrap();
+    let test_filter = TestFilterBuilder::new(
+        RunIgnored::Only,
+        None,
+        TestFilterPatterns::default(),
+        vec![expr],
+    )
+    .unwrap();
+
+    let test_list = FIXTURE_TARGETS.make_test_list(
+        "with-timeout-retries-success",
+        &test_filter,
+        &TargetRunner::empty(),
+    )?;
+    let config = load_config();
+    let profile = config
+        .profile("with-timeout-retries-success")
+        .expect("with-timeout-retries-success config is valid");
+    let build_platforms = BuildPlatforms::new_with_no_target().unwrap();
+    let profile = profile.apply_build_platforms(&build_platforms);
+
+    let profile_retries = profile.retries();
+    assert_eq!(
+        profile_retries,
+        RetryPolicy::new_without_delay(2),
+        "retries set in with-timeout-retries-success profile"
+    );
+
+    let runner = TestRunnerBuilder::default()
+        .build(
+            &test_list,
+            &profile,
+            vec![],
+            SignalHandlerKind::Noop,
+            InputHandlerKind::Noop,
+            DoubleSpawnInfo::disabled(),
+            TargetRunner::empty(),
+        )
+        .unwrap();
+
+    let (instance_statuses, _run_stats) = execute_collect(runner);
+
+    // With retries and on-timeout=pass, timed out tests should not be retried
+    for test_name in [
+        "test_slow_timeout",
+        "test_slow_timeout_2",
+        "test_slow_timeout_subprocess",
+    ] {
+        let (_, instance_value) = instance_statuses
+            .iter()
+            .find(|&(&(_, name), _)| name == test_name)
+            .unwrap_or_else(|| panic!("{test_name} should be present"));
+
+        match &instance_value.status {
+            InstanceStatus::Skipped(_) => panic!("{test_name} should have been run"),
+            InstanceStatus::Finished(run_statuses) => {
+                assert_eq!(
+                    run_statuses.len(),
+                    1,
+                    "{test_name} should have been run exactly once \
+                     (timed out tests that pass are not retried)",
+                );
+
+                let status = run_statuses.last_status();
+                assert_eq!(
+                    status.result,
+                    ExecutionResult::Timeout {
+                        result: SlowTimeoutResult::Pass
+                    }
+                );
+            }
+        };
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_timeout_with_flaky() -> Result<()> {
+    test_init();
+
+    let pcx = ParseContext::new(&PACKAGE_GRAPH);
+    let expr = Filterset::parse(
+        "test(test_flaky_slow_timeout_mod_3)".to_owned(),
+        &pcx,
+        FiltersetKind::Test,
+    )
+    .unwrap();
+    let test_filter = TestFilterBuilder::new(
+        RunIgnored::Only,
+        None,
+        TestFilterPatterns::default(),
+        vec![expr],
+    )
+    .unwrap();
+
+    let test_list = FIXTURE_TARGETS.make_test_list(
+        "with-timeout-retries-success",
+        &test_filter,
+        &TargetRunner::empty(),
+    )?;
+    let config = load_config();
+    let profile = config
+        .profile("with-timeout-retries-success")
+        .expect("with-timeout-retries-success config is valid");
+    let build_platforms = BuildPlatforms::new_with_no_target().unwrap();
+    let profile = profile.apply_build_platforms(&build_platforms);
+
+    let runner = TestRunnerBuilder::default()
+        .build(
+            &test_list,
+            &profile,
+            vec![],
+            SignalHandlerKind::Noop,
+            InputHandlerKind::Noop,
+            DoubleSpawnInfo::disabled(),
+            TargetRunner::empty(),
+        )
+        .unwrap();
+
+    let (instance_statuses, _run_stats) = execute_collect(runner);
+
+    let (_, instance_value) = instance_statuses
+        .iter()
+        .find(|&(&(_, name), _)| name == "test_flaky_slow_timeout_mod_3")
+        .unwrap_or_else(|| panic!("test_flaky_slow_timeout_mod_3 should be present"));
+
+    match &instance_value.status {
+        InstanceStatus::Skipped(_) => panic!("test_flaky_slow_timeout_mod_3 should have been run"),
+        InstanceStatus::Finished(run_statuses) => {
+            assert!(
+                run_statuses.len() == 3,
+                "test_flaky_slow_timeout_mod_3 should have been run 3 times, was run {} times",
+                run_statuses.len()
+            );
+
+            match run_statuses.describe() {
+                ExecutionDescription::Flaky {
+                    last_status,
+                    prior_statuses,
+                } => {
+                    for (i, prior_status) in prior_statuses.iter().enumerate() {
+                        assert!(
+                            matches!(prior_status.result, ExecutionResult::Fail { .. }),
+                            "prior attempt {} should be fail, got {:?}",
+                            i + 1,
+                            prior_status.result
+                        );
+                    }
+                    assert!(
+                        matches!(
+                            last_status.result,
+                            ExecutionResult::Timeout {
+                                result: SlowTimeoutResult::Pass
+                            }
+                        ),
+                        "last attempt should be a successful timeout, was {:?}",
+                        last_status.result
+                    );
+                }
+                _ => panic!("test_flaky_slow_timeout_mod_3 should be flaky"),
+            }
+        }
+    };
+
+    Ok(())
+}
+
 #[test]
 fn test_list_tests() -> Result<()> {
     test_init();
@@ -705,7 +882,7 @@ fn test_termination() -> Result<()> {
         .unwrap();
 
     let (instance_statuses, run_stats) = execute_collect(runner);
-    assert_eq!(run_stats.timed_out, 3, "3 tests timed out");
+    assert_eq!(run_stats.failed_timed_out, 3, "3 tests timed out");
     for test_name in [
         "test_slow_timeout",
         "test_slow_timeout_2",
@@ -733,7 +910,10 @@ fn test_termination() -> Result<()> {
                     "{test_name} should have taken less than 5 seconds, actually took {:?}",
                     run_status.time_taken
                 );
-                run_status.result == ExecutionResult::Timeout
+                run_status.result
+                    == ExecutionResult::Timeout {
+                        result: SlowTimeoutResult::Fail,
+                    }
             }
         };
         if !valid {
@@ -746,3 +926,107 @@ fn test_termination() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_override_timeout_result() -> Result<()> {
+    test_init();
+
+    let pcx = ParseContext::new(&PACKAGE_GRAPH);
+    let expr = Filterset::parse(
+        "test(/^test_slow_timeout/)".to_owned(),
+        &pcx,
+        FiltersetKind::Test,
+    )
+    .unwrap();
+    let test_filter = TestFilterBuilder::new(
+        RunIgnored::Only,
+        None,
+        TestFilterPatterns::default(),
+        vec![expr],
+    )
+    .unwrap();
+
+    let test_list = FIXTURE_TARGETS.make_test_list(
+        "with-timeout-success",
+        &test_filter,
+        &TargetRunner::empty(),
+    )?;
+    let config = load_config();
+    let profile = config
+        .profile("with-timeout-success")
+        .expect("with-timeout-success config is valid");
+    let build_platforms = BuildPlatforms::new_with_no_target().unwrap();
+    let profile = profile.apply_build_platforms(&build_platforms);
+
+    let runner = TestRunnerBuilder::default()
+        .build(
+            &test_list,
+            &profile,
+            vec![],
+            SignalHandlerKind::Noop,
+            InputHandlerKind::Noop,
+            DoubleSpawnInfo::disabled(),
+            TargetRunner::empty(),
+        )
+        .unwrap();
+
+    let (instance_statuses, run_stats) = execute_collect(runner);
+
+    println!("{instance_statuses:?}");
+    assert_eq!(run_stats.finished_count, 3, "3 tests should have finished");
+    assert_eq!(
+        run_stats.passed_timed_out, 1,
+        "1 test should pass with timeout"
+    );
+    assert_eq!(run_stats.failed_timed_out, 2, "2 tests should fail");
+
+    for test_name in [
+        "test_slow_timeout",
+        "test_slow_timeout_2",
+        "test_slow_timeout_subprocess",
+    ] {
+        let (_, instance_value) = instance_statuses
+            .iter()
+            .find(|&(&(_, name), _)| name == test_name)
+            .unwrap_or_else(|| panic!("{test_name} should be present"));
+        let (expected, actual) = match &instance_value.status {
+            InstanceStatus::Skipped(_) => panic!("{test_name} should have been run"),
+            InstanceStatus::Finished(run_statuses) => {
+                assert_eq!(
+                    run_statuses.len(),
+                    1,
+                    "{test_name} should have been run exactly once",
+                );
+                let run_status = run_statuses.last_status();
+                assert!(
+                    run_status.time_taken < Duration::from_secs(5),
+                    "{test_name} should have taken less than 5 seconds, actually took {:?}",
+                    run_status.time_taken
+                );
+                if matches!(test_name, "test_slow_timeout") {
+                    (
+                        ExecutionResult::Timeout {
+                            result: SlowTimeoutResult::Pass,
+                        },
+                        run_status.result,
+                    )
+                } else {
+                    (
+                        ExecutionResult::Timeout {
+                            result: SlowTimeoutResult::Fail,
+                        },
+                        run_status.result,
+                    )
+                }
+            }
+        };
+
+        assert_eq!(
+            expected, actual,
+            "for test_slow_timeout, mismatch in status: expected {:?}, actual {:?}",
+            expected, actual
+        );
+    }
+
+    Ok(())
+}
